/*! ********************************************
 * SimpleSequencer.java
 * Copyright 2022 masahiro takamoto (MotChan)
 * Distributed under the MIT license. 
 * see -> license.txt
 ******************************************** */
import sora.graybox.GrayBox;
import sora.graybox.IGrayBoxModule;
import sora.graybox.MidiEvent;
import sora.graybox.ProcessData;
// シーケンサーモジュール
public class SimpleSequencer implements IGrayBoxModule {
	private int _note = 72;  // ノートナンバー
	private int _frameCount = 0;  // フレームカウンター
	private int _sampleRate = 44100;  // サンプリング周波数
	
	public SimpleSequencer() {}
// sora.graybox.IGrayBoxModule -->
	// ホストのサンプリングレートが変更された時に呼ばれるメソッド
	// GrayBox.setSampleRate() -> このメソッド
	public void updateSampleRate(float sampleRate) {
		_sampleRate = (int)sampleRate;
		if (_frameCount >= _sampleRate)_frameCount = 0;
	}
	// ホストのオーディオチャンネル数が変更された時に呼ばれるメソッド
	// GrayBox.setAudioChannels() -> このメソッド
	public void updateAudioChannels(int ch){}
	// ホストのプロセスブロックサイズが変更された時に呼ばれるメソッド
	// GrayBox.setProcessBlockSize() -> このメソッド
	public void updateProcessBlockSize(int size){}
	// ホストのGrayBox.resetModules()メソッドが使用された時に呼ばれるメソッド
	// GrayBox.resetModules() -> このメソッド
	public void resetModule(){ _frameCount = 0; _note = 72; }
	// ホストのGrayBox.initiModules()メソッドが使用された時に呼ばれるメソッド
	// GrayBox.initiModules() -> このメソッド
	public void initiModule(){ _frameCount = 0; _note = 72; }
	// このモジュールがホストへ接続された時に呼ばれるメソッド
	// GrayBox.attachModule() -> このメソッド
	public void connectModule(GrayBox host) {
		updateSampleRate(host.getSampleRate());
	}
	// このモジュールがホストから取り外された時に呼ばれるメソッド
	// GrayBox.detachModule() -> このメソッド
	public void disconnectModule(){}
	// このモジュールが接続されているスロットの、バイパス状況が更新された時に呼ばれるメソッド
	// GrayBox.bypassProcess() -> このメソッド
	public void updateBypassStatus(boolean bypass){}
	// ホストのGrayBox.startProcess()メソッドが使用された時に呼ばれるメソッド
	// GrayBox.startProcess() -> このメソッド
	public void startProcess(){}
	// ホストのGrayBox.stopProcess()メソッドが使用された時に呼ばれるメソッド
	// GrayBox.stopProcess() -> このメソッド
	public void stopProcess(){}
	// ホストのGrayBox.suspendProcess()メソッドが使用された時に呼ばれるメソッド
	// GrayBox.suspendProcess() -> このメソッド
	public void suspendProcess(){}
	// ホストのGrayBox.resumeProcess()メソッドが使用された時に呼ばれるメソッド
	// GrayBox.resumeProcess() -> このメソッド
	public void resumeProcess(){}
	// ホストが音声処理を実行した時に呼ばれるメソッド
	// GrayBox.process() -> このメソッド
	public void process(ProcessData procData) {
		// ProcessDataオブジェクトはホストから音声処理に必要な情報を受け取り、
		// 各モジュールへ伝えるオブジェクトです。
		// GrayBoxシステムにおけるMIDIメッセージの伝送はProcessDataオブジェクトが持つ
		// MIDIイベントリスト(片方向リンクリスト)を使用して行われます。
		// MIDIメッセージの送信側はこのリストにMIDIイベントを登録していき、
		// MIDIメッセージの受信側はこのリストからMIDIイベント参照してメッセージの処理を行います。
		
		// ・ProcessData.midiEventList
		//   MIDIイベントリスト(先頭ノード(リスト管理用ダミーノード))
		//   このリストのノードの並びはMidiEvent.offsetFramesの昇順です。
		// ・ProcessData.addToMidiEventList(event:MidiEvent, check:Boolean = false)
		//   MIDIイベントリストへイベントを登録します。(通常はこちらを使用してください)
		//   第1引数 リストに登録するMidiEventオブジェクト
		//   第2引数 登録しようとするMidiEventオブジェクトが
		//           リストに既に登録されているかをチェックする
	// MIDIイベント生成、登録 -->
		int _frame = procData.offsetFrames; // 開始フレーム
		int _frameEnd = _frame + procData.sampleFrames; // 終了フレーム
		for ( ; _frame < _frameEnd; _frame++) {
			// 1秒ごとにMIDIイベント生成 -->
			if (_frameCount++ >= _sampleRate) {
			// ノートオフメッセージ (前回の分)
				MidiEvent _midiEvent = new MidiEvent();  // MIDIイベント生成
				_midiEvent.type = 0x80;  // ノートオフ
				_midiEvent.channel = 0;  // Midiチャンネル
				_midiEvent.value0 = _note;  // ノートナンバー
				_midiEvent.offsetFrames = _frame; // このイベントの発生フレーム
				procData.addToMidiEventList(_midiEvent); // MIDIイベントリストへ登録
			// ノートナンバー更新
				if (++_note > 72)_note = 60;
			// ノートオンメッセージ (今回の分)
				_midiEvent = new MidiEvent();  // MIDIイベント生成
				_midiEvent.type = 0x90; // ノートオン
				_midiEvent.channel = 0; // Midiチャンネル
				_midiEvent.value0 = _note; // ノートナンバー
				_midiEvent.value1 = 127; // ノートオンベロシティー
				_midiEvent.offsetFrames = _frame; // このイベントの発生フレーム
				procData.addToMidiEventList(_midiEvent); // MIDIイベントリストへ登録
				// フレームカウンターリセット
				_frameCount = 0;
			}
		}
	}
}